@@ -7,14 +7,65 @@ import robin_stocks.stocks as stocks
 import robin_stocks.urls as urls
 
 
+@helper.login_required
+def get_historical_portfolio(interval=None, span='week', bounds='regular',info=None):
+    interval_check = ['5minute', '10minute', 'hour', 'day', 'week']
+    span_check = ['day', 'week', 'month', '3month', 'year', '5year', 'all']
+    bounds_check = ['extended', 'regular', 'trading']
+
+    if interval not in interval_check:
+        if interval is None and (bounds != 'regular' and span != 'all'):
+            print ('ERROR: Interval must be None for "all" span "regular" bounds')
+            return ([None])
+        print(
+            'ERROR: Interval must be "5minute","10minute","hour","day",or "week"')
+        return([None])
+    if span not in span_check:
+        print('ERROR: Span must be "day","week","month","3month","year",or "5year"')
+        return([None])
+    if bounds not in bounds_check:
+        print('ERROR: Bounds must be "extended","regular",or "trading"')
+        return([None])
+    if (bounds == 'extended' or bounds == 'trading') and span != 'day':
+        print('ERROR: extended and trading bounds can only be used with a span of "day"')
+        return([None])
+
+    account = profiles.load_account_profile('account_number')
+    url = urls.portfolis_historicals(account)
+    payload = {
+        'interval': interval,
+        'span': span,
+        'bounds': bounds
+    }
+    data = helper.request_get(url, 'regular', payload)
+
+    return(helper.filter(data, info))
+
 @helper.login_required
 def get_all_positions(info=None):
     """Returns a list containing every position ever traded.
 
     :param info: Will filter the results to get a specific value.
     :type info: Optional[str]
-    :returns: Returns a list of dictionaries of key/value pairs for each ticker. If info parameter is provided, \
+    :returns: [list] Returns a list of dictionaries of key/value pairs for each ticker. If info parameter is provided, \
     a list of strings is returned where the strings are the value of the key that matches info.
+    :Dictionary Keys: * url
+                      * instrument
+                      * account
+                      * account_number
+                      * average_buy_price
+                      * pending_average_buy_price
+                      * quantity
+                      * intraday_average_buy_price
+                      * intraday_quantity
+                      * shares_held_for_buys
+                      * shares_held_for_sells
+                      * shares_held_for_stock_grants
+                      * shares_held_for_options_collateral
+                      * shares_held_for_options_events
+                      * shares_pending_from_options_events
+                      * updated_at
+                      * created_at
 
     """
     url = urls.positions()
@@ -24,13 +75,30 @@ def get_all_positions(info=None):
 
 
 @helper.login_required
-def get_current_positions(info=None):
-    """Returns a list of stocks/options that are currently held.
+def get_open_stock_positions(info=None):
+    """Returns a list of stocks that are currently held.
 
     :param info: Will filter the results to get a specific value.
     :type info: Optional[str]
-    :returns: Returns a list of dictionaries of key/value pairs for each ticker. If info parameter is provided, \
+    :returns: [list] Returns a list of dictionaries of key/value pairs for each ticker. If info parameter is provided, \
     a list of strings is returned where the strings are the value of the key that matches info.
+    :Dictionary Keys: * url
+                      * instrument
+                      * account
+                      * account_number
+                      * average_buy_price
+                      * pending_average_buy_price
+                      * quantity
+                      * intraday_average_buy_price
+                      * intraday_quantity
+                      * shares_held_for_buys
+                      * shares_held_for_sells
+                      * shares_held_for_stock_grants
+                      * shares_held_for_options_collateral
+                      * shares_held_for_options_events
+                      * shares_pending_from_options_events
+                      * updated_at
+                      * created_at
 
     """
     url = urls.positions()
@@ -47,8 +115,22 @@ def get_dividends(info=None):
 
     :param info: Will filter the results to get a specific value.
     :type info: Optional[str]
-    :returns: Returns a list of dictionaries of key/value pairs for each divident payment. If info parameter is provided, \
+    :returns: [list] Returns a list of dictionaries of key/value pairs for each divident payment. If info parameter is provided, \
     a list of strings is returned where the strings are the value of the key that matches info.
+    :Dictionary Keys: * id
+                      * url
+                      * account
+                      * instrument
+                      * amount
+                      * rate
+                      * position
+                      * withholding
+                      * record_date
+                      * payable_date
+                      * paid_at
+                      * state
+                      * nra_withholding
+                      * drip_enabled
 
     """
     url = urls.dividends()
@@ -69,7 +151,7 @@ def get_total_dividends():
 
     dividend_total = 0
     for item in data:
-        dividend_total += float(item['amount'])
+        dividend_total += float(item['amount']) if (item['state'] == 'paid' or item['state'] == 'reinvested') else 0
     return(dividend_total)
 
 
@@ -229,6 +311,19 @@ def get_bank_transfers(info=None):
     data = helper.request_get(url, 'pagination')
     return(helper.filter(data, info))
 
+@helper.login_required
+def get_card_transactions(info=None):
+    """Returns all debit card transactions made on the account
+
+    :param info: Will filter the results to get a specific value. 'direction' gives if it was debit or credit. 
+    :type info: Optional[str]
+    :returns: Returns a list of dictionaries of key/value pairs for each transfer. If info parameter is provided, \
+    a list of strings is returned where the strings are the value of the key that matches info.
+
+    """
+    url = urls.cardtransactions()
+    data = helper.request_get(url, 'pagination')
+    return(helper.filter(data, info))
 
 @helper.login_required
 def get_stock_loan_payments(info=None):
@@ -426,7 +521,7 @@ def get_all_watchlists(info=None):
 
     """
     url = urls.watchlists()
-    data = helper.request_get(url, 'pagination')
+    data = helper.request_get(url, 'result')
     return(helper.filter(data, info))
 
 
@@ -441,8 +536,16 @@ def get_watchlist_by_name(name='Default', info=None):
     :returns: Returns a list of dictionaries that contain the instrument urls and a url that references itself.
 
     """
+
+    #Get id of requested watchlist
+    all_watchlists = get_all_watchlists()
+    watchlist_id = ''
+    for wl in all_watchlists['results']:
+        if wl['display_name'] == name:
+            watchlist_id = wl['id']
+
     url = urls.watchlists(name)
-    data = helper.request_get(url, 'pagination')
+    data = helper.request_get(url,'list_id',{'list_id':watchlist_id})
     return(helper.filter(data, info))
 
 
@@ -479,21 +582,12 @@ def delete_symbols_from_watchlist(inputSymbols, name='Default'):
 
     """
     symbols = helper.inputs_to_set(inputSymbols)
-    symbols = stocks.get_fundamentals(symbols, info='instrument')
+    ids = stocks.get_instruments_by_symbols(symbols, info='id')
+    data = []
 
-    watchlist = get_watchlist_by_name(name=name)
-
-    items = []
-    data = None
-
-    for symbol in symbols:
-        for list_ in watchlist:
-            if symbol == list_['instrument']:
-                items.append(symbol[37:])
-
-    for item in items:
-        url = urls.watchlists() + name + item
-        data = helper.request_delete(url)
+    for id in ids:
+        url = urls.watchlist_delete(name, id)
+        data.append(helper.request_delete(url))
 
     return(data)
 
@@ -509,7 +603,7 @@ def build_holdings(with_dividends=False):
     percentage of portfolio, and average buy price.
 
     """
-    positions_data = get_current_positions()
+    positions_data = get_open_stock_positions()
     portfolios_data = profiles.load_portfolio_profile()
     accounts_data = profiles.load_account_profile()
 
