@@ -10,8 +10,23 @@ def get_quotes(inputSymbols, info=None):
     :type inputSymbols: str or list
     :param info: Will filter the results to have a list of the values that correspond to key that matches info.
     :type info: Optional[str]
-    :returns: If info parameter is left as None then the list will contain a dictionary of key/value pairs for each ticker. \
+    :returns: [list] If info parameter is left as None then the list will contain a dictionary of key/value pairs for each ticker. \
     Otherwise, it will be a list of strings where the strings are the values of the key that corresponds to info.
+    :Dictionary Keys: * ask_price
+                      * ask_size
+                      * bid_price
+                      * bid_size
+                      * last_trade_price
+                      * last_extended_hours_trade_price
+                      * previous_close
+                      * adjusted_previous_close
+                      * previous_close_date
+                      * symbol
+                      * trading_halted
+                      * has_traded
+                      * last_trade_price_source
+                      * updated_at
+                      * instrument
 
     """
     symbols = helper.inputs_to_set(inputSymbols)
@@ -39,8 +54,32 @@ def get_fundamentals(inputSymbols, info=None):
     :type inputSymbols: str or list
     :param info: Will filter the results to have a list of the values that correspond to key that matches info.
     :type info: Optional[str]
-    :returns: If info parameter is left as None then the list will contain a dictionary of key/value pairs for each ticker. \
+    :returns: [list] If info parameter is left as None then the list will contain a dictionary of key/value pairs for each ticker. \
     Otherwise, it will be a list of strings where the strings are the values of the key that corresponds to info.
+    :Dictionary Keys: * open
+                      * high
+                      * low
+                      * volume
+                      * average_volume_2_weeks
+                      * average_volume
+                      * high_52_weeks
+                      * dividend_yield
+                      * float
+                      * low_52_weeks
+                      * market_cap
+                      * pb_ratio
+                      * pe_ratio
+                      * shares_outstanding
+                      * description
+                      * instrument
+                      * ceo
+                      * headquarters_city
+                      * headquarters_state
+                      * sector
+                      * industry
+                      * num_employees
+                      * year_founded
+                      * symbol
 
     """
     symbols = helper.inputs_to_set(inputSymbols)
@@ -70,8 +109,32 @@ def get_instruments_by_symbols(inputSymbols, info=None):
     :type inputSymbols: str or list
     :param info: Will filter the results to have a list of the values that correspond to key that matches info.
     :type info: Optional[str]
-    :returns: If info parameter is left as None then the list will contain a dictionary of key/value pairs for each ticker. \
+    :returns: [list] If info parameter is left as None then the list will a dictionary of key/value pairs for each ticker. \
     Otherwise, it will be a list of strings where the strings are the values of the key that corresponds to info.
+    :Dictionary Keys: * id
+                      * url
+                      * quote
+                      * fundamentals
+                      * splits
+                      * state
+                      * market
+                      * simple_name
+                      * name
+                      * tradeable
+                      * tradability
+                      * symbol
+                      * bloomberg_unique
+                      * margin_initial_ratio
+                      * maintenance_ratio
+                      * country
+                      * day_trade_ratio
+                      * list_date
+                      * min_tick_size
+                      * type
+                      * tradable_chain_id
+                      * rhs_tradability
+                      * fractional_tradability
+                      * default_collar_fraction
 
     """
     symbols = helper.inputs_to_set(inputSymbols)
@@ -98,8 +161,32 @@ def get_instrument_by_url(url, info=None):
     :type url: str
     :param info: Will filter the results to have a list of the values that correspond to key that matches info.
     :type info: Optional[str]
-    :returns: If info parameter is left as None then the list will contain a dictionary of key/value pairs for each ticker. \
-    Otherwise, it will be a list of strings where the strings are the values of the key that corresponds to info.
+    :returns: [dict or str] If info parameter is left as None then will return a dictionary of key/value pairs for a specific url. \
+    Otherwise, it will be the string value of the key that corresponds to info.
+    :Dictionary Keys: * id
+                      * url
+                      * quote
+                      * fundamentals
+                      * splits
+                      * state
+                      * market
+                      * simple_name
+                      * name
+                      * tradeable
+                      * tradability
+                      * symbol
+                      * bloomberg_unique
+                      * margin_initial_ratio
+                      * maintenance_ratio
+                      * country
+                      * day_trade_ratio
+                      * list_date
+                      * min_tick_size
+                      * type
+                      * tradable_chain_id
+                      * rhs_tradability
+                      * fractional_tradability
+                      * default_collar_fraction
 
     """
     data = helper.request_get(url, 'regular')
@@ -115,7 +202,7 @@ def get_latest_price(inputSymbols, includeExtendedHours=True):
     :param includeExtendedHours: Leave as True if you want to get extendedhours price if available. \
     False if you only want regular hours price, even after hours.
     :type includeExtendedHours: bool
-    :returns: A list of prices as strings.
+    :returns: [list] A list of prices as strings.
 
     """
     symbols = helper.inputs_to_set(inputSymbols)
@@ -123,10 +210,13 @@ def get_latest_price(inputSymbols, includeExtendedHours=True):
 
     prices = []
     for item in quote:
-        if item['last_extended_hours_trade_price'] is None or not includeExtendedHours:
-            prices.append(item['last_trade_price'])
+        if item:
+            if item['last_extended_hours_trade_price'] is None or not includeExtendedHours:
+                prices.append(item['last_trade_price'])
+            else:
+                prices.append(item['last_extended_hours_trade_price'])
         else:
-            prices.append(item['last_extended_hours_trade_price'])
+            prices.append(None)
     return(prices)
 
 
@@ -136,7 +226,7 @@ def get_name_by_symbol(symbol):
 
     :param symbol: The ticker of the stock as a string.
     :type symbol: str
-    :returns: Returns the simple name of the stock. If the simple name does not exist then returns the full name.
+    :returns: [str] Returns the simple name of the stock. If the simple name does not exist then returns the full name.
 
     """
     try:
@@ -164,7 +254,7 @@ def get_name_by_url(url):
 
     :param url: The url of the stock as a string.
     :type url: str
-    :returns: Returns the simple name of the stock. If the simple name does not exist then returns the full name.
+    :returns: [str] Returns the simple name of the stock. If the simple name does not exist then returns the full name.
 
     """
     data = helper.request_get(url)
@@ -184,13 +274,13 @@ def get_symbol_by_url(url):
 
     :param url: The url of the stock as a string.
     :type url: str
-    :returns: Returns the ticker symbol of the stock.
+    :returns: [str] Returns the ticker symbol of the stock.
 
     """
     data = helper.request_get(url)
     return helper.filter(data, info='symbol')
 
-
+@helper.convert_none_to_string
 def get_ratings(symbol, info=None):
     """Returns the ratings for a stock, including the number of buy, hold, and sell ratings.
 
@@ -199,9 +289,12 @@ def get_ratings(symbol, info=None):
     :param info: Will filter the results to contain a dictionary of values that correspond to the key that matches info. \
     Possible values are summary, ratings, and instrument_id
     :type info: Optional[str]
-    :returns: If info parameter is left as None then the list will contain a dictionary of key/value pairs for each ticker. \
-    Otherwise, it will contain the values that correspond to the keyword that matches info. In this case, \
-    the value will also be a dictionary.
+    :returns: [dict] If info parameter is left as None then the list will contain a dictionary of key/value pairs for each ticker. \
+    Otherwise, it will contain the values that correspond to the keyword that matches info.
+    :Dictionary Keys: * summary - value is a dictionary
+                      * ratings - value is a list of dictionaries
+                      * instrument_id - value is a string
+                      * ratings_published_at - value is a string
 
     """
     try:
@@ -212,6 +305,9 @@ def get_ratings(symbol, info=None):
 
     url = urls.ratings(symbol)
     data = helper.request_get(url)
+    if not data:
+        return(data)
+
     if (len(data['ratings']) == 0):
         return(data)
     else:
@@ -221,7 +317,7 @@ def get_ratings(symbol, info=None):
 
     return(helper.filter(data, info))
 
-
+@helper.convert_none_to_string
 def get_popularity(symbol, info=None):
     """Returns the number of open positions.
 
@@ -229,8 +325,10 @@ def get_popularity(symbol, info=None):
     :type symbol: str
     :param info: Will filter the results to be a string value.
     :type info: Optional[str]
-    :returns: If the info parameter is provided, then the function will extract the value of the key \
+    :returns: [dict] If the info parameter is provided, then the function will extract the value of the key \
     that matches the info parameter. Otherwise, the whole dictionary is returned.
+    :Dictionary Keys: * instrument
+                      * num_open_positions
 
     """
     try:
@@ -246,14 +344,32 @@ def get_popularity(symbol, info=None):
 
 
 def get_events(symbol, info=None):
-    """Returns the events related to a stock.
+    """Returns the events related to a stock that the user owns. For example, if you owned options for USO and that stock \
+    underwent a stock split resulting in you owning shares of newly created USO1, then that event will be returned when calling \
+    get_events('uso1')
 
     :param symbol: The stock ticker.
     :type symbol: str
     :param info: Will filter the results to get a specific value.
     :type info: Optional[str]
-    :returns: If the info parameter is provided, then the function will extract the value of the key \
+    :returns: [list] If the info parameter is provided, then the function will extract the value of the key \
     that matches the info parameter. Otherwise, the whole dictionary is returned.
+    :Dictionary Keys: * account
+                      * cash_component
+                      * chain_id
+                      * created_at
+                      * direction
+                      * equity_components
+                      * event_date
+                      * id
+                      * option
+                      * position
+                      * quantity
+                      * state
+                      * total_cash_amount
+                      * type
+                      * underlying_price
+                      * updated_at
 
     """
     try:
@@ -276,9 +392,16 @@ def get_earnings(symbol, info=None):
     :type symbol: str
     :param info: Will filter the results to get a specific value.
     :type info: Optional[str]
-    :returns: Returns a list of dictionaries. If info parameter is provided, \
+    :returns: [list] Returns a list of dictionaries. If info parameter is provided, \
     a list of strings is returned where the strings are the value \
     of the key that matches info.
+    :Dictionary Keys: * symbol
+                      * instrument
+                      * year
+                      * quarter
+                      * eps
+                      * report
+                      * call
 
     """
     try:
@@ -301,9 +424,24 @@ def get_news(symbol, info=None):
     :type symbol: str
     :param info: Will filter the results to get a specific value.
     :type info: Optional[str]
-    :returns: Returns a list of dictionaries. If info parameter is provided, \
+    :returns: [list] Returns a list of dictionaries. If info parameter is provided, \
     a list of strings is returned where the strings are the value \
     of the key that matches info.
+    :Dictionary Keys: * api_source
+                      * author
+                      * num_clicks
+                      * preview_image_url
+                      * published_at
+                      * relay_url
+                      * source
+                      * summary
+                      * title
+                      * updated_at
+                      * url
+                      * uuid
+                      * related_instruments
+                      * preview_text
+                      * currency_id
 
     """
     try:
@@ -326,9 +464,14 @@ def get_splits(symbol, info=None):
     :param info: Will filter the results to get a specific value. Possible options are \
     url, instrument, execution_date, divsor, and multiplier.
     :type info: Optional[str]
-    :returns: Returns a list of dictionaries. If info parameter is provided, \
+    :returns: [list] Returns a list of dictionaries. If info parameter is provided, \
     a list of strings is returned where the strings are the value \
     of the key that matches info.
+    :Dictionary Keys: * url
+                      * instrument
+                      * execution_date
+                      * multiplier
+                      * divisor
 
     """
     try:
@@ -347,7 +490,31 @@ def find_instrument_data(query):
 
     :param query: The keyword to search for.
     :type query: str
-    :returns: Returns a list of dictionaries that contain the instrument data for each stock that matches the query.
+    :returns: [list] Returns a list of dictionaries that contain the instrument data for each stock that matches the query.
+    :Dictionary Keys: * id
+                      * url
+                      * quote
+                      * fundamentals
+                      * splits
+                      * state
+                      * market
+                      * simple_name
+                      * name
+                      * tradeable
+                      * tradability
+                      * symbol
+                      * bloomberg_unique
+                      * margin_initial_ratio
+                      * maintenance_ratio
+                      * country
+                      * day_trade_ratio
+                      * list_date
+                      * min_tick_size
+                      * type
+                      * tradable_chain_id
+                      * rhs_tradability
+                      * fractional_tradability
+                      * default_collar_fraction
 
     """
     url = urls.instruments()
@@ -363,21 +530,40 @@ def find_instrument_data(query):
         return(data)
 
 
-def get_historicals(inputSymbols, span='week', bounds='regular'):
-    """Represents the data that is used to make the graphs.
+def get_stock_historicals(inputSymbols, interval='hour', span='week', bounds='regular', info=None):
+    """Represents the historicl data for a stock.
 
     :param inputSymbols: May be a single stock ticker or a list of stock tickers.
     :type inputSymbols: str or list
+    :param interval: Interval to retrieve data for. Values are '5minute', '10minute', 'hour', 'day', 'week'. Default is 'hour'.
+    :type interval: Optional[str]
     :param span: Sets the range of the data to be either 'day', 'week', 'month', '3month', 'year', or '5year'. Default is 'week'.
     :type span: Optional[str]
-    :param bounds: Represents if graph will include extended trading hours or just regular trading hours. Values are 'extended' or 'regular'.
+    :param bounds: Represents if graph will include extended trading hours or just regular trading hours. Values are 'extended' or 'regular'. Default is 'regular'
     :type bounds: Optional[str]
-    :returns: Returns a list of dictionaries where each dictionary is for a different time. If multiple stocks are provided \
+    :param info: Will filter the results to have a list of the values that correspond to key that matches info.
+    :type info: Optional[str]
+    :returns: [list] Returns a list of dictionaries where each dictionary is for a different time. If multiple stocks are provided \
     the historical data is listed one after another.
+    :Dictionary Keys: * begins_at
+                      * open_price
+                      * close_price
+                      * high_price
+                      * low_price
+                      * volume
+                      * session
+                      * interpolated
+                      * symbol
 
     """
+    interval_check = ['5minute', '10minute', 'hour', 'day', 'week']
     span_check = ['day', 'week', 'month', '3month', 'year', '5year']
     bounds_check = ['extended', 'regular', 'trading']
+
+    if interval not in interval_check:
+        print(
+            'ERROR: Interval must be "5minute","10minute","hour","day",or "week"')
+        return([None])
     if span not in span_check:
         print('ERROR: Span must be "day","week","month","3month","year",or "5year"')
         return([None])
@@ -388,19 +574,6 @@ def get_historicals(inputSymbols, span='week', bounds='regular'):
         print('ERROR: extended and trading bounds can only be used with a span of "day"')
         return([None])
 
-    if span == 'day':
-        interval = '5minute'
-    elif span == 'week':
-        interval = '10minute'
-    elif span == 'month':
-        interval = 'hour'
-    elif span == '3month':
-        interval = 'hour'
-    elif span == 'year':
-        interval = 'day'
-    else:
-        interval = 'week'
-
     symbols = helper.inputs_to_set(inputSymbols)
     url = urls.historicals()
     payload = {'symbols': ','.join(symbols),
@@ -422,7 +595,7 @@ def get_historicals(inputSymbols, span='week', bounds='regular'):
             subitem['symbol'] = stockSymbol
             histData.append(subitem)
 
-    return(histData)
+    return(helper.filter(histData, info))
 
 
 def get_stock_quote_by_id(stock_id, info=None):
@@ -431,10 +604,26 @@ def get_stock_quote_by_id(stock_id, info=None):
 
     :param stock_id: robinhood stock id
     :type stock_id: str
-    :param info: Will filter the results to get a specific value. Possible options are url, instrument, execution_date,
+    :param info: Will filter the results to get a specific value. Possible options are url, instrument, execution_date, \
     divsor, and multiplier.
     :type info: Optional[str]
-    :return:
+    :return: [dict] If the info parameter is provided, then the function will extract the value of the key \
+    that matches the info parameter. Otherwise, the whole dictionary is returned.
+    :Dictionary Keys: * ask_price
+                      * ask_size
+                      * bid_price
+                      * bid_size
+                      * last_trade_price
+                      * last_extended_hours_trade_price
+                      * previous_close
+                      * adjusted_previous_close
+                      * previous_close_date
+                      * symbol
+                      * trading_halted
+                      * has_traded
+                      * last_trade_price_source
+                      * updated_at
+                      * instrument
     """
     url = urls.marketdata_quotes(stock_id)
     data = helper.request_get(url)
@@ -448,10 +637,26 @@ def get_stock_quote_by_symbol(symbol, info=None):
 
     :param symbol: robinhood stock id
     :type stock_id: str
-    :param info: Will filter the results to get a specific value. Possible options are url, instrument, execution_date,
+    :param info: Will filter the results to get a specific value. Possible options are url, instrument, execution_date, \
     divsor, and multiplier.
     :type info: Optional[str]
-    :return:
+    :return: [dict] If the info parameter is provided, then the function will extract the value of the key \
+    that matches the info parameter. Otherwise, the whole dictionary is returned.
+    :Dictionary Keys: * ask_price
+                      * ask_size
+                      * bid_price
+                      * bid_size
+                      * last_trade_price
+                      * last_extended_hours_trade_price
+                      * previous_close
+                      * adjusted_previous_close
+                      * previous_close_date
+                      * symbol
+                      * trading_halted
+                      * has_traded
+                      * last_trade_price_source
+                      * updated_at
+                      * instrument
     """
 
     return get_stock_quote_by_id(helper.id_for_stock(symbol))
@@ -463,10 +668,10 @@ def get_pricebook_by_id(stock_id, info=None):
 
     :param stock_id: robinhood stock id
     :type stock_id: str
-    :param info: Will filter the results to get a specific value. Possible options are url, instrument, execution_date,
+    :param info: Will filter the results to get a specific value. Possible options are url, instrument, execution_date, \
     divsor, and multiplier.
     :type info: Optional[str]
-    :return:
+    :return: Returns a dictionary of asks and bids.
 
     """
     url = urls.marketdata_pricebook(stock_id)
@@ -481,7 +686,7 @@ def get_pricebook_by_symbol(symbol, info=None):
 
     :param symbol: symbol id
     :type symbol: str
-    :param info: Will filter the results to get a specific value. Possible options are url, instrument, execution_date,
+    :param info: Will filter the results to get a specific value. Possible options are url, instrument, execution_date, \
     divsor, and multiplier.
     :type info: Optional[str]
     :return: Returns a dictionary of asks and bids.
